/***************************************************************************************
 * @file     reb_main.c
 * @version  V1.0
 * @date     31. March 2016
 *
 * @note
 * VORAGO Technologies 
 *
 * @note
 * Copyright (c) 2013-2016 VORAGO Technologies. 
 *
 * @par
 * BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND BY 
 * ALL THE TERMS AND CONDITIONS OF THE VORAGO TECHNOLOGIES END USER LICENSE AGREEMENT. 
 * THIS SOFTWARE IS PROVIDED "AS IS". NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY 
 * AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. VORAGO TECHNOLOGIES 
 * SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR CONSEQUENTIAL 
 * DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 ****************************************************************************************/
#include <stdio.h>
#include "va108xx.h"
#include "reb_log.h"
#include "reb_board.h"
#include "reb_adt75.h"
#include "driver_common.h"
#include "reb_max11619.h"
#include "reb_timer.h"
#include "gpio_va108xx.h"

//   *******  Global variables set here  *******
volatile uint8_t bounce_cnt = 0; 
         uint32_t G_TDO[32], EF_ADD_value[32]  ;   // storage for TDO data for each message    



//*********************************************************************************************
//    Project intended to drive JTAG port on another VA108xx MCU 
//     Port pin use table:
//     PA0 - TCK   (output)
//		 PA1 - TMS    (output)
//     PA2 - TRST   (output)
//     PA3 - TDI    (output)
//     PA4 - TDO    (input)


//*********************************************************************************************

#define sTCK   0
#define sTMS   1
#define sTRST  2
#define sTDI   3 
#define sTDO	 4 





//   ****** subroutine to cycle TCK 1k times
//   clock will be high upon entry and exit. 

void TCK_1k(void)  
{
volatile	    uint32_t  cnt,cnt2   ; 
			
		for(cnt=0; cnt < 1000; cnt ++)
		{
			VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTCK);  // set tclk low 
	           for(cnt2=0; cnt2 < 10; cnt2 ++)  { __NOP() ;}   
	    VOR_GPIO->BANK[0].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT shou
						 for(cnt2=0; cnt2 < 10; cnt2 ++)  { __NOP() ;} 
		}
	    
	return ; 
}

//   **  short delay  

void tclk_delay(void)
{
uint32_t  i  ;  

for(i=1;i<50;i++ )     
	{
			__NOP() ;  
	}//  delay here 

} 
//*********************************************************************************************
//    Subroutine to setup port pins and RESET the JTAG controller  
//        setup PA0-3 as output and PA4 as an input
//         drive reset pin of JTAG low for two TCLK cycles 
//         Move state machine to pause parking position (1 more clock with TMS = 0 
//         Data changes on falling edge of TCK and is sampled on rising edge. 


int32_t Reset_JTAG()  
{
	uint32_t i   ;  
		VOR_GPIO->BANK[0].DIR |= (1 << sTCK)|(1 << sTMS)|(1 << sTRST)|(1 << sTDI) ; 
		VOR_GPIO->BANK[0].DIR &= ~(1<<sTDO)  ;  
	
		VOR_GPIO->BANK[0].DATAOUT |= (1 << sTCK)|(1 << sTMS)|(1 << sTRST)|(1 << sTDI);  // set all outputs to high state
	
		VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTRST);  // set trst low 
		tclk_delay()  ; 
		VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTCK);   // set tclk low 
	  tclk_delay()  ;
	  VOR_GPIO->BANK[0].DATAOUT |= (1 << sTCK);    // set tclk high -  DUT should process RST active here
		tclk_delay() ;  
	   VOR_GPIO->BANK[0].DATAOUT |= (1<<sTRST)  ;  	//  set TRST high 
	   VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTCK);   // set tclk low 
		VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTMS);    // set TMS low 
	  tclk_delay()  ;
	  VOR_GPIO->BANK[0].DATAOUT |= (1 << sTCK);     // set tclk high
	  tclk_delay()  ;
	  tclk_delay()  ;  //   add second delay just for debug purposes to show break
	return 0  ;  
}




uint32_t rev_bit_order (uint32_t num_bits, uint32_t in_data) 
{
volatile	  uint32_t out_data, temp1, mask   ; 
	   temp1 = 0  ;
	   mask = 1 ; 
	  out_data = 0   ;  
	  while( num_bits -- > 0 ) 
	  {
	  temp1 = (in_data & mask) ; 
		if(temp1 != 0)
			{
				 out_data |= 0x1  ;  //  set lsb to 1 
			} 
     out_data <<=  1 ;  //  shift left 
		 mask <<= 1   ;    //   shift mask 1 position left
	  }
//	  out_data >>= 1   ;   //   correct for one too many shifts left.  
	
 
   return out_data >> 1   ;  	
	 
}


//*********************************************************************************************
//    Subroutine to shift out data to data register of JTAG
//    input characters:  bitcnt - number of bits to shift  (1 to 65) 
//                       TDI_value - value to be shifted  (note:  LSBit is shifted out first)  
//    output character:  TDO_data -  This is the information clocked in on the TDO line during the shift.  
//
//     variable explanation:  out3 is a three bit field that consists of  TMS, TRST, TDI. This gets put on port pins on falling edge of tck. 
//    
//
//
//
//


uint32_t SDR_sub(uint32_t bitcnt, uint32_t TDI_value) 
{
volatile	 int32_t  TDO_data, TDO_bitpos, temp1  ; 
volatile		 int8_t  bit_out,out3  ;  
    TDO_data = 0  ;  	
		out3 = 0  ;  
	  TDO_bitpos = 1   ;  
    temp1 = TDI_value  ;  
	
//  ** Move JTAG state machine position  (TMS =  1 0 0 to get from park to point where data is shifted) 

		  VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTCK);  // set tclk low 
			VOR_GPIO->BANK[0].DATAOUT |= (1 << sTMS);   
	    tclk_delay()  ;
	    VOR_GPIO->BANK[0].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
			tclk_delay()  ;	
			VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTCK);  // set tclk low 
				 VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTMS); 		 
//	VOR_GPIO->BANK[0].DATAOUT |= (1 << sTMS);   
	    tclk_delay()  ;
	    VOR_GPIO->BANK[0].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
			tclk_delay()  ;	
			VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTCK);  // set tclk low 
			 VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTMS);   
	    tclk_delay()  ;
	    VOR_GPIO->BANK[0].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
      tclk_delay()  ;
// ** Start shifting data out here  
	while (bitcnt-- > 0) 
	{
		  bit_out = TDI_value & 0x0001    ;
			TDI_value = TDI_value >> 1  ;    //   shift TDI_value one position to get ready for next bit
			if(bit_out == 1) 
			{
			       VOR_GPIO->BANK[0].DATAOUT |= (1 << sTDI);
			}
			else
			{
				    VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTDI); 
				
			}
       if(bitcnt == 0)  {		  VOR_GPIO->BANK[0].DATAOUT |= (1 << sTMS); } // set TMS high on last bit 
		  VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTCK);  // set tclk low 
 
	    tclk_delay()  ; 
	    VOR_GPIO->BANK[0].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
			temp1 = (VOR_GPIO->BANK[0].DATAIN & (1<<sTDO))  ;  //  sample TDO on rising edge of TCLK 
			if(temp1 != 0)
			{
				TDO_data |= TDO_bitpos ;  //  set bit in TDO if pin sample = 1, otherwise leave as zero 
			}
		
			TDO_bitpos = (  TDO_bitpos << 1 ) ;   //  move bit position to left 
		  tclk_delay() ;			
	}
	
	//  ** Move JTAG state machine position  (TMS =  1 0 to get from point where data is shifted to park) 

//		  VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTCK);  // set tclk low 
//			 VOR_GPIO->BANK[0].DATAOUT |= (1 << sTMS);   
//	    tclk_delay()  ;
//	    VOR_GPIO->BANK[0].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
//		   tclk_delay()  ;
	     VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTCK);  // set tclk low 
			 VOR_GPIO->BANK[0].DATAOUT |= (1 << sTMS);   
	    tclk_delay()  ;
	    VOR_GPIO->BANK[0].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
			 tclk_delay()  ;
	     VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTCK);  // set tclk low 
			 VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTMS);   
	    tclk_delay()  ;
	    VOR_GPIO->BANK[0].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
       tclk_delay()  ;
		return TDO_data ;  
}  


//  ****  trying to figure out why return statements are being optimized out  

uint32_t SIR_sub(uint32_t bitcnt, uint32_t TDI_value) 
{
volatile	 int32_t  TDO_data, TDO_bitpos, temp1  ; 
volatile		 int8_t  bit_out,out3  ;  
    TDO_data = 0  ;  	
		out3 = 0  ;  
	  TDO_bitpos = 1   ;  
    temp1 = TDI_value  ;  
	
//  ** Move JTAG state machine position  (TMS =  1 1 0 0 to get from park to point where data is shifted) 

		  VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTCK);  // set tclk low 
			VOR_GPIO->BANK[0].DATAOUT |= (1 << sTMS);   
	    tclk_delay()  ;
	    VOR_GPIO->BANK[0].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
			tclk_delay()  ;	
			VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTCK);  // set tclk low 
			 VOR_GPIO->BANK[0].DATAOUT |= (1 << sTMS);   
	    tclk_delay()  ;
	    VOR_GPIO->BANK[0].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
			tclk_delay()  ;	
			VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTCK);  // set tclk low 
			 VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTMS);   
	    tclk_delay()  ;
	    VOR_GPIO->BANK[0].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
      tclk_delay()  ;
			VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTCK);  // set tclk low 
			 VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTMS);   
	    tclk_delay()  ;
	    VOR_GPIO->BANK[0].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
      tclk_delay()  ;
// ** Start shifting data out here  
	while (bitcnt-- > 0) 
	{
		  bit_out = TDI_value & 0x0001    ;
			TDI_value = TDI_value >> 1  ;    //   shift TDI_value one position to get ready for next bit
			if(bit_out == 1) 
			{
			       VOR_GPIO->BANK[0].DATAOUT |= (1 << sTDI);
			}
			else
			{
				    VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTDI); 
				
			}
      if(bitcnt == 0)  {		  VOR_GPIO->BANK[0].DATAOUT |= (1 << sTMS); } // set TMS high on last bit 
		  VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTCK);  // set tclk low 
 
	    tclk_delay()  ; 
	    VOR_GPIO->BANK[0].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
			temp1 = (VOR_GPIO->BANK[0].DATAIN & (1<<sTDO))  ;  //  sample TDO on rising edge of TCLK 
			if(temp1 != 0)
			{
				TDO_data |= TDO_bitpos ;  //  set bit in TDO if pin sample = 1, otherwise leave as zero 
			}
		
			TDO_bitpos = (  TDO_bitpos << 1 ) ;   //  move bit position to left 
		  tclk_delay() ;			
	}
	
	//  ** Move JTAG state machine position  (TMS =   1 0 to get from point where data is shifted to park) 

//		  VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTCK);  // set tclk low 
//			 VOR_GPIO->BANK[0].DATAOUT |= (1 << sTMS);   
//	    tclk_delay()  ;
//	    VOR_GPIO->BANK[0].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
//		   tclk_delay()  ;
	     VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTCK);  // set tclk low 
			 VOR_GPIO->BANK[0].DATAOUT |= (1 << sTMS);   
	    tclk_delay()  ;
	    VOR_GPIO->BANK[0].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
			 tclk_delay()  ;
	     VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTCK);  // set tclk low 
			 VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTMS);   
	    tclk_delay()  ;
	    VOR_GPIO->BANK[0].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
       tclk_delay()  ;
		return TDO_data ;  
}  

//  ***  end of try to optimize out 



#define EF_ADDR 0x14
#define EF_WDATA 0x15
#define EF_RDATA 0x16
#define EF_TIMING 0x17
#define EF_CMD 0x18
#define EF_STATUS 0x19

//  *******************************************************************
//  Read efuse 
//    calling parameters  (count = number of location to read (0-31), start_add = start address (0-31)
//  Per programmers guide 
//      - Load EF_WDATA with 0 
//			- Load EF_ADDR with desired read address
//			- Load EF_CMD with 0x1 
//      - Poll EF_STATUS for Bit0 to be 1 
//      - Load EF_CMD with 0x5  
//			- Read EF_RDATA 
//			- Load EF_CMD with 0x0 to disable the interface.  


uint32_t Read_EF_ADR(uint32_t add_count, uint32_t start_add)  

{
		uint8_t  HBO_ready, cnt=0, loop_count  ; 
volatile	   uint32_t read_add   ;  
	  loop_count = 0 ; 
	 while(loop_count++ < add_count) 
	 {
	  G_TDO[0] = SIR_sub(9,EF_WDATA)  ; 
	  G_TDO[1] = SDR_sub(32,0x0) ;
	
		read_add = (start_add+loop_count-1)  ;  
		G_TDO[2] = SIR_sub(9,EF_ADDR)  ; 
	  G_TDO[3] = SDR_sub(6,read_add) ;
	
		G_TDO[20] = SIR_sub(9,EF_STATUS)  ; 
	  G_TDO[21] = SDR_sub(3,0x1) ;
		
	
		G_TDO[4] = SIR_sub(9,EF_CMD)  ; 
	  G_TDO[5] = SDR_sub(4,0x1) ;
	
	
		HBO_ready = 0 ; 
	  while(HBO_ready == 0) 
		{
		G_TDO[5] = SIR_sub(9,EF_STATUS)  ; 
	  G_TDO[6] = SDR_sub(3,0x1) ;
		if(G_TDO[6] != 0)  HBO_ready = 1 ;
     cnt ++ ;  			
		}
	
		G_TDO[7] = SIR_sub(9,EF_CMD)  ; 
	  G_TDO[8] = SDR_sub(4,0x5) ;
	
		G_TDO[9] = SIR_sub(9,EF_RDATA)  ; 
	  G_TDO[10] = SDR_sub(32,0x0) ;
		EF_ADD_value[loop_count-1] = G_TDO[10]  ;  

		G_TDO[11] = SIR_sub(4,EF_CMD)  ; 
	  G_TDO[12] = SDR_sub(32,0x0) ;	
  }		 
    return cnt  ;  
	
}

//  *******************************************************************
//  Program efuse 
//    calling parameters  (address (0-31, value(32bits, any combination)


//  Must have TCLK select before starting the sequence  
//  Per programmers guide 
//       
//			Step 1 - Load EF_ADDR with desired write address "address"
//      Step 2 - Load EF_WDATA with value to be programmed "value"
//      Step 3  - Load EF_TIMING with proper timing 
//			Step 4 - Load EF_CMD with 0x3 
//      Step 5- Poll EF_STATUS for Bit0 to be 1 
//      Step 6- bring write enable pin low
//      Step 7 - Load EF_CMD with 0x7 
//      Step 8- Poll EF_STATUS for Bit1 to be 0
//      Step 9- bring efuse write enable pin high
//			Step 10- Load EF_CMD with 0x0 to disable the interface. 
//      Step 11 - read the location to verify program worked 

uint32_t Prog_EF_ADR(uint32_t address, uint32_t value)  
{
		uint8_t  HBO_ready, cnt=0,ctrl_busy  ; 
volatile	   uint32_t read_add   ;  

		 SIR_sub(9,0x0009)  ;   // set TCLK as source clock
		 SDR_sub(4,0x1) ; 
				
		G_TDO[2] = SIR_sub(9,EF_ADDR)  ;  //  step 1
	  G_TDO[3] = SDR_sub(6, address) ;
	
		G_TDO[0] = SIR_sub(9,EF_WDATA)  ;  //  step 2
	  G_TDO[1] = SDR_sub(33,value) ;
	
		SIR_sub(9,EF_TIMING)  ; //  step 3
		SDR_sub(26,0x840022) ;   //  set timing for efuse operation 	
	
		G_TDO[4] = SIR_sub(9,EF_CMD)  ; // step 4
	  G_TDO[5] = SDR_sub(4,0x3) ;
	
		HBO_ready = 0 ;    // step5  
	  while(HBO_ready == 0) 
		{
		G_TDO[5] = SIR_sub(9,EF_STATUS)  ; //Step 6
	  G_TDO[6] = SDR_sub(3,0x1) ;
		if(G_TDO[6] != 0)  HBO_ready = 1 ;
     cnt ++ ;  			
		}
	
		G_TDO[7] = SIR_sub(9,EF_CMD)  ;   //Step 8
	  G_TDO[8] = SDR_sub(4,0x7) ;
		
				
		ctrl_busy = 1 ; 
	  while(ctrl_busy == 1)  //Step 9
		{
		G_TDO[20] = SIR_sub(9,EF_STATUS)  ; 
	  G_TDO[21] = SDR_sub(3,0x1) ;
			if(!(G_TDO[21] & 0x2)) ctrl_busy = 0 ; 
		} 

		G_TDO[11] = SIR_sub(4,EF_CMD)  ; //step 10 
	  G_TDO[12] = SDR_sub(32,0x0) ;	
  	 
    return cnt  ;  
}



int main()
{
	uint8_t cur_temp = 0; 
	int32_t ret_code = 0;
	uint16_t adc_data = 0;
volatile	uint32_t TDO_string[12],i,k,bill,k2,crypt[10]  ;  
	
	/* Enable clock for all peripherals */ 
	VOR_SYSCONFIG->PERIPHERAL_CLK_ENABLE = ( CLK_ENABLE_PORTA | CLK_ENABLE_PORTB | CLK_ENABLE_SPIA | CLK_ENABLE_SPIB | CLK_ENABLE_SPIC |
																					 CLK_ENABLE_UARTA | CLK_ENABLE_UARTB | CLK_ENABLE_I2CA | CLK_ENABLE_I2CB | CLK_ENABLE_IRQSEL |
																					 CLK_ENABLE_IOMGR | CLK_ENABLE_UTILITY | CLK_ENABLE_PORTIO | CLK_ENABLE_SYSTEM ); 
	

 
	  k = rev_bit_order( 9,0x01053)  ;
	    crypt[4] = k   ;  

		
	
	
		Reset_JTAG()  	  ;  
	
	

	
//	   TDO_string[10] = SIR_sub(9,0x101) ;
//		 TDO_string[11] = SDR_sub(9,0x101) ;

/*
	
	  TDO_string[9] = SDR_sub(32,0x00) ;  //  read out TAP ID # which is available after a RESET
		
		TDO_string[3] = SIR_sub(9,0x011)  ; 
		TDO_string[4] = SDR_sub(10,0x1d7) ;   //  set all three POR bits to reset part
		TDO_string[5] = SDR_sub(10,0x1d0) ;   //  clear POR bits 
		
*/
		
		TDO_string[6] = SIR_sub(9,0x0009)  ; 
		TDO_string[7] = SDR_sub(4,0x1) ; 
		
	//			TCK_1k()  ;  //  wait approximately 1msec for POR to release
				
		TDO_string[8] = SIR_sub(9,EF_TIMING)  ; 
		TDO_string[9] = SDR_sub(26,0x840022) ;   //  set timing for efuse operation  
		
	/*		
	  TDO_string[3] = SIR_sub(9,(0x1E0 |0x06))  ; 
		TDO_string[4] = SDR_sub(32,0x00) ;    
 	
		k2 = SDR_sub(10,0x1d7) ; 
		TDO_string[1] = rev_bit_order( 32,k2)  ;   
		k2 = SDR_sub(10,0x1d0) ; 
		TDO_string[2] = rev_bit_order( 32,k2)  ; 

     TCK_1k()  ;   //  wait around for 1k tclk cycles 
*/	 
    bill =   Read_EF_ADR(16,0); 
		
		k2 = Prog_EF_ADR(6,3) ;  
			
	  bill =   Read_EF_ADR(16,0); 
		
		while(1)  ;    // hang here during code debug phase  
	
	
#ifdef GPIO_DRIVER
	VOR_DRIVER_GPIO *PA10 = &Driver_GPIOA_10; 
#endif 	
	
	
	/* Initialize Logging module */ 
	VOR_Log_Init(PORTA_UARTA, LOG_DBG); 
	VOR_Log(LOG_INFO, ANSI_COLOR_GREEN"Welcome to Vorago Technologies REB1 VA108xx software! \n");
	if( VOR_SYSCONFIG->PROCID == PROC_ID_TORCH )
		VOR_Log(LOG_INFO, ANSI_COLOR_GREEN"VA10820 MCU ...\n");
	else
		VOR_Log(LOG_INFO, ANSI_COLOR_GREEN"VA10800 MCU ... \n");
	VOR_Log(LOG_INFO, "Restarted... \n");
	VOR_Log(LOG_INFO, "Reset Status: 0x%08x \n", VOR_SYSCONFIG->RST_STAT); 
	
	/* Configure PA[10]. This GPIO will continously blink led D2. */ 	
#ifdef GPIO_DRIVER
	PA10->Initialize(NULL);
	PA10->Control(VOR_GPIO_CONTROL_MASK, VOR_GPIO_CTRL_ENABLE); 
	PA10->Control(VOR_GPIO_CONTROL_DIR, VOR_GPIO_DIR_OUTPUT); 
	PA10->Write(1);
#else 
	VOR_GPIO->BANK[0].DIR |= (1 << PORTA_10_D2); 
	VOR_GPIO->BANK[0].DATAMASK |= (1 << PORTA_10_D2);
	VOR_GPIO->BANK[0].DATAOUT &= ~(1 << PORTA_10_D2); 
#endif 	
	/* Configure PA[11]. Input to blink led D3. */ 
	VOR_GPIO->BANK[0].DIR &= ~(1 << PORTA_11_SW_USER); 
	VOR_GPIO->BANK[0].DATAMASK |= (1 << PORTA_11_SW_USER);
	
	/* Configure PA[7]. This GPIO will turn ON led D3. */ 
	VOR_GPIO->BANK[0].DIR |= (1 << PORTA_7_D3); 
	VOR_GPIO->BANK[0].DATAMASK |= (1 << PORTA_7_D3);
	VOR_GPIO->BANK[0].DATAOUT |= (1 << PORTA_7_D3); 
	
	/* Intialize MSec and Sec Timers */
	ret_code = VOR_TIM0_MSecInitialize(100); 	
	if( ret_code )
		VOR_Log(LOG_ERROR, "MSec Timer initialization failed! \n");

	ret_code = VOR_TIM1_SecInitialize(1); 	
	if( ret_code )
		VOR_Log(LOG_ERROR, "Sec Timer initialization failed! \n");
	
	/* Initialize temperature sensor - ADT75 */ 
	ret_code = VOR_ADT75_Initialize(); 
	if( ret_code )
		VOR_Log(LOG_ERROR, "ADT75 initialization failed! \n");

	/* Initialize SPI interface to Maxim ADC - MAX11619 */ 
	ret_code = VOR_MAX11619_Initialize(); 
	if( ret_code )
		VOR_Log(LOG_ERROR, "MAX11619 initialization failed! \n");
	
	while(1) {
		if( VOR_TIM1_SecExpired() ) {
			/* Retrieve current temperature */ 
			ret_code = VOR_ADT75_GetTemp(&cur_temp);
			if( ret_code )
				VOR_Log(LOG_ERROR, "ADT75 read failed! \n");
			else
				VOR_Log(LOG_INFO, "\nTemperature: %d(C)\n", cur_temp); 
		
			/* Configure Maxim ADC */ 
			ret_code = VOR_MAX11619_Configure();
			if( ret_code )
				VOR_Log(LOG_ERROR, "MAX11619 configuration failed! \n");
		
			/* Retrieve sampled data from ADC */ 
			ret_code = VOR_MAX11619_GetData(&adc_data);	
			if( ret_code )
				VOR_Log(LOG_ERROR, "MAX11619 get data failed! \n"); 
			else
				VOR_Log(LOG_INFO, "\nADC O/P (0(0V) to 1023(3.3V)): %d\n", adc_data);  
			
			VOR_TIM1_SecClear(); 	
		}	
		if( VOR_TIM0_MSecExpired() ) {
#ifdef GPIO_DRIVER			
			PA10->Write(1); 
#else
			VOR_GPIO->BANK[0].DATAOUT ^= (1 << PORTA_10_D2);		
#endif 			
			if( !(VOR_GPIO->BANK[0].DATAIN & (1 << PORTA_11_SW_USER)) )
				bounce_cnt++; 
			
			if(bounce_cnt >= 2) {
				VOR_GPIO->BANK[0].DATAOUT ^= (1 << PORTA_7_D3); 
				bounce_cnt = 0; 
			}
			VOR_TIM0_MSecClear(); 
		}
	}
}
